﻿using System.Collections;
using System.Collections.Specialized;
using System.Linq;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Interactivity;

namespace TestProject
{
    /// <summary>
    /// 리스트 박스 선택 동작
    /// </summary>
    public class ListBoxSelectionBehavior : Behavior<ListBox>
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Field
        ////////////////////////////////////////////////////////////////////////////////////////// Static
        //////////////////////////////////////////////////////////////////////////////// Public

        #region 선택 항목 리스트 속성 - SelectedItemsProperty

        /// <summary>
        /// 선택 항목 리스트 속성
        /// </summary>
        public static readonly DependencyProperty SelectedItemsProperty = DependencyProperty.Register
        (
            nameof(SelectedItems),
            typeof(IList),
            typeof(ListBoxSelectionBehavior),
            new FrameworkPropertyMetadata
            (
                null,
                FrameworkPropertyMetadataOptions.BindsTwoWayByDefault,
                SelectedItemsPropertyChangedCallback
            )
        );

        #endregion
        #region 선택 값 리스트 속성 - SelectedValuesProperty

        /// <summary>
        /// 선택 값 리스트 속성
        /// </summary>
        public static readonly DependencyProperty SelectedValuesProperty = DependencyProperty.Register
        (
            nameof(SelectedValues),
            typeof(IList),
            typeof(ListBoxSelectionBehavior),
            new FrameworkPropertyMetadata
            (
                null,
                FrameworkPropertyMetadataOptions.BindsTwoWayByDefault,
                SelectedValuesPropertyChangedCallback
            )
        );

        #endregion

        ////////////////////////////////////////////////////////////////////////////////////////// Instance
        //////////////////////////////////////////////////////////////////////////////// Private

        #region Field

        /// <summary>
        /// 뷰 처리 여부
        /// </summary>
        private bool viewHandled;

        /// <summary>
        /// 모델 처리 여부
        /// </summary>
        private bool modelHandled;

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Property
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 선택 항목 리스트 - SelectedItems

        /// <summary>
        /// 선택 항목 리스트
        /// </summary>
        public IList SelectedItems
        {
            get => (IList)GetValue(SelectedItemsProperty);
            set => SetValue(SelectedItemsProperty, value);
        }

        #endregion
        #region 선택 값 리스트 - SelectedValues

        /// <summary>
        /// 선택 값 리스트
        /// </summary>
        public IList SelectedValues
        {
            get => (IList)GetValue(SelectedValuesProperty);
            set => SetValue(SelectedValuesProperty, value);
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Static
        //////////////////////////////////////////////////////////////////////////////// Private

        #region 선택 항목 리스트 속성 변경시 콜백 처리하기 - SelectedItemsPropertyChangedCallback(d, e)

        /// <summary>
        /// 선택 항목 리스트 속성 변경시 콜백 처리하기
        /// </summary>
        /// <param name="d">의존 객체</param>
        /// <param name="e">이벤트 인자</param>
        private static void SelectedItemsPropertyChangedCallback(DependencyObject d, DependencyPropertyChangedEventArgs e)
        {
            ListBoxSelectionBehavior behavior = d as ListBoxSelectionBehavior;

            if(behavior.modelHandled)
            {
                return;
            }

            if(behavior.AssociatedObject == null)
            {
                return;
            }

            behavior.modelHandled = true;

            behavior.UpdateSelectedValues();

            behavior.SelectItems();

            behavior.modelHandled = false;
        }

        #endregion
        #region 선택 값 리스트 속성 변경시 콜백 처리하기 - SelectedValuesPropertyChangedCallback(d, e)

        /// <summary>
        /// 선택 값 리스트 속성 변경시 콜백 처리하기
        /// </summary>
        /// <param name="d">의존 객체</param>
        /// <param name="e">이벤트 인자</param>
        private static void SelectedValuesPropertyChangedCallback(DependencyObject d, DependencyPropertyChangedEventArgs e)
        {
            ListBoxSelectionBehavior behavior = d as ListBoxSelectionBehavior;

            if(behavior.modelHandled)
            {
                return;
            }

            if(behavior.AssociatedObject == null)
            {
                return;
            }

            behavior.modelHandled = true;

            behavior.UpdateSelectedItems();

            behavior.SelectItems();

            behavior.modelHandled = false;
        }

        #endregion
        #region 속성 값 구하기 - GetPropertyValue(source, path)

        /// <summary>
        /// 속성 값 구하기
        /// </summary>
        /// <param name="source">소스 객체</param>
        /// <param name="path">경로</param>
        /// <returns>속성 값</returns>
        private static object GetPropertyValue(object source, string path)
        {
            if(string.IsNullOrWhiteSpace(path))
            {
                return source;
            }

            while(true)
            {
                if(path.Contains('.'))
                {
                    string[] pathItemArray = path.Split('.');

                    string remainingProperty = path.Substring(path.IndexOf('.') + 1);

                    source = source?.GetType().GetProperty(pathItemArray[0])?.GetValue(source, null);

                    path = remainingProperty;

                    continue;
                }

                return source?.GetType().GetProperty(path)?.GetValue(source, null);
            }
        }

        #endregion

        ////////////////////////////////////////////////////////////////////////////////////////// Instance
        //////////////////////////////////////////////////////////////////////////////// Protected
        ////////////////////////////////////////////////////////////////////// Function

        #region 접착시 처리하기 - OnAttached()

        /// <summary>
        /// 접착시 처리하기
        /// </summary>
        protected override void OnAttached()
        {
            base.OnAttached();

            AssociatedObject.SelectionChanged += ListBox_SelectionChanged;

            ((INotifyCollectionChanged)AssociatedObject.Items).CollectionChanged += ListBox_CollectionChanged;

            this.modelHandled = true;

            UpdateSelectedItems();

            SelectItems();

            this.modelHandled = false;
        }

        #endregion
        #region 탈착시 처리하기 - OnDetaching()

        /// <summary>
        /// 탈착시 처리하기
        /// </summary>
        protected override void OnDetaching()
        {
            base.OnDetaching();

            if(AssociatedObject != null)
            {
                AssociatedObject.SelectionChanged -= ListBox_SelectionChanged;

                ((INotifyCollectionChanged)AssociatedObject.Items).CollectionChanged -= ListBox_CollectionChanged;
            }
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////// Private
        ////////////////////////////////////////////////////////////////////// Event

        #region 리스트 박스 선택 변경시 처리하기 - ListBox_SelectionChanged(sender, e)

        /// <summary>
        /// 리스트 박스 선택 변경시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void ListBox_SelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            if(this.viewHandled)
            {
                return;
            }

            if(AssociatedObject.Items.SourceCollection == null)
            {
                return;
            }

            SelectedItems = AssociatedObject.SelectedItems.Cast<object>().ToObservableCollection();
        }

        #endregion
        #region 리스트 박스 컬렉션 변경시 처리하기 - ListBox_CollectionChanged(sender, e)

        /// <summary>
        /// 리스트 박스 컬렉션 변경시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void ListBox_CollectionChanged(object sender, NotifyCollectionChangedEventArgs e)
        {
            if(this.viewHandled)
            {
                return;
            }

            if(AssociatedObject.Items.SourceCollection == null)
            {
                return;
            }

            SelectItems();
        }

        #endregion

        ////////////////////////////////////////////////////////////////////// Function

        #region 항목 선택하기 - SelectItems()

        /// <summary>
        /// 항목 선택하기
        /// </summary>
        /// <remarks>속성을 기준으로 리스트 박스에서 항목을 선택합니다</remarks>
        private void SelectItems()
        {
            this.viewHandled = true;

            AssociatedObject.SelectedItems.Clear();

            if(SelectedItems != null)
            {
                foreach(object item in SelectedItems)
                {
                    AssociatedObject.SelectedItems.Add(item);
                }
            }

            this.viewHandled = false;
        }

        #endregion
        #region 선택 항목 리스트 업데이트하기 - UpdateSelectedItems()

        /// <summary>
        /// 선택 항목 리스트 업데이트하기
        /// </summary>
        /// <remarks>SelectedValues 속성값을 기반으로 SelectedItems 속성값을 업데이트 한다.</remarks>
        private void UpdateSelectedItems()
        {
            if(SelectedValues == null)
            {
                SelectedItems = null;
            }
            else
            {
                SelectedItems = AssociatedObject.Items.Cast<object>()
                    .Where(i => SelectedValues.Contains(GetPropertyValue(i, AssociatedObject.SelectedValuePath)))
                    .ToArray();
            }
        }

        #endregion
        #region 선택 값 리스트 업데이트하기 - UpdateSelectedValues()

        /// <summary>
        /// 선택 값 리스트 업데이트하기
        /// </summary>
        /// <remarks>SelectedItems 속성값을 기반으로 SelectedValues 속성값을 업데이트한다.</remarks>
        private void UpdateSelectedValues()
        {
            if(SelectedItems == null)
            {
                SelectedValues = null;
            }
            else
            {
                SelectedValues = SelectedItems.Cast<object>()
                        .Select(i => GetPropertyValue(i, AssociatedObject.SelectedValuePath))
                        .ToArray();
            }
        }

        #endregion
    }
}